@@ -1,14 +1,11 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2015 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 #![doc = include_str!("../README.md")]
 #![allow(unknown_lints)]
 #![deny(missing_docs)]
 #![deny(rustdoc::broken_intra_doc_links)]
 #![allow(clippy::upper_case_acronyms)]
+#![allow(clippy::non_send_fields_in_send_ty)]
 #![allow(renamed_and_removed_lints)]
 #![allow(unknown_lints)]
 
@@ -54,6 +51,14 @@ cfg_if::cfg_if! {
     } else if #[cfg(any(target_os = "linux", target_os = "android"))] {
         mod linux;
         use linux as sys;
+        pub(crate) mod users;
+
+        #[cfg(test)]
+        pub(crate) const MIN_USERS: usize = 1;
+    } else if #[cfg(target_os = "freebsd")] {
+        mod freebsd;
+        use freebsd as sys;
+        pub(crate) mod users;
 
         #[cfg(test)]
         pub(crate) const MIN_USERS: usize = 1;
@@ -67,8 +72,8 @@ cfg_if::cfg_if! {
 }
 
 pub use common::{
-    get_current_pid, AsU32, DiskType, DiskUsage, Gid, LoadAvg, NetworksIter, Pid, ProcessStatus,
-    RefreshKind, Signal, Uid, User,
+    get_current_pid, AsU32, DiskType, DiskUsage, Gid, LoadAvg, NetworksIter, Pid,
+    ProcessRefreshKind, ProcessStatus, RefreshKind, Signal, Uid, User,
 };
 pub use sys::{Component, Disk, NetworkData, Networks, Process, Processor, System};
 pub use traits::{
@@ -289,7 +294,7 @@ mod test {
         // We don't want to test on unsupported systems.
         if System::IS_SUPPORTED {
             let s = System::new();
-            assert!(!s.host_name().expect("Failed to get host name").is_empty());
+            assert!(s.host_name().is_some());
         }
     }
 
@@ -321,33 +326,31 @@ mod test {
 
     #[test]
     fn check_processors_number() {
-        let s = System::new();
+        let mut s = System::new();
 
+        // This information isn't retrieved by default.
+        assert!(s.processors().is_empty());
         if System::IS_SUPPORTED {
-            assert!(!s.processors().is_empty());
-            // In case we are running inside a VM, it's possible to not have a physical core, only
-            // logical ones, which is why we don't test `physical_cores_count > 0`.
+            // The physical cores count is recomputed every time the function is called, so the
+            // information must be relevant even with nothing initialized.
             let physical_cores_count = s
                 .physical_core_count()
                 .expect("failed to get number of physical cores");
-            assert!(physical_cores_count <= s.processors().len());
+
+            s.refresh_cpu();
+            // The processors shouldn't be empty anymore.
+            assert!(!s.processors().is_empty());
+
+            // In case we are running inside a VM, it's possible to not have a physical core, only
+            // logical ones, which is why we don't test `physical_cores_count > 0`.
+            let physical_cores_count2 = s
+                .physical_core_count()
+                .expect("failed to get number of physical cores");
+            assert!(physical_cores_count2 <= s.processors().len());
+            assert_eq!(physical_cores_count, physical_cores_count2);
         } else {
-            assert!(s.processors().is_empty());
             assert_eq!(s.physical_core_count(), None);
         }
         assert!(s.physical_core_count().unwrap_or(0) <= s.processors().len());
     }
 }
-
-// Used to check that System is Send and Sync.
-#[cfg(doctest)]
-/// ```
-/// fn is_send<T: Send>() {}
-/// is_send::<sysinfo::System>();
-/// ```
-///
-/// ```
-/// fn is_sync<T: Sync>() {}
-/// is_sync::<sysinfo::System>();
-/// ```
-pub mod check_if_system_is_send_and_sync {}
