@@ -1,24 +1,19 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2015 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use crate::sys::component::{self, Component};
 use crate::sys::disk;
 use crate::sys::process::*;
 use crate::sys::processor::*;
 use crate::sys::utils::get_all_data;
-use crate::{Disk, LoadAvg, Networks, Pid, ProcessExt, RefreshKind, SystemExt, User};
+use crate::{Disk, LoadAvg, Networks, Pid, ProcessRefreshKind, RefreshKind, SystemExt, User};
 
-use libc::{self, c_char, sysconf, _SC_HOST_NAME_MAX, _SC_PAGESIZE};
+use libc::{self, c_char, sysconf, _SC_CLK_TCK, _SC_HOST_NAME_MAX, _SC_PAGESIZE};
 use std::collections::HashMap;
 use std::fs::File;
 use std::io::{BufRead, BufReader, Read};
 use std::path::Path;
 use std::str::FromStr;
 use std::sync::{Arc, Mutex};
-use std::time::SystemTime;
 
 // This whole thing is to prevent having too many files open at once. It could be problematic
 // for processes using a lot of files and using sysinfo at the same time.
@@ -121,6 +116,12 @@ pub struct System {
     networks: Networks,
     users: Vec<User>,
     boot_time: u64,
+    /// Field set to `false` in `update_processors` and to `true` in `refresh_processes_specifics`.
+    ///
+    /// The reason behind this is to avoid calling the `update_processors` more than necessary.
+    /// For example when running `refresh_all` or `refresh_specifics`.
+    need_processors_update: bool,
+    clock_cycle: u64,
 }
 
 impl System {
@@ -133,30 +134,38 @@ impl System {
         self.processors.len() as f32 * 100.
     }
 
-    fn clear_procs(&mut self) {
-        self.refresh_processors(true);
+    fn clear_procs(&mut self, refresh_kind: ProcessRefreshKind) {
+        let (total_time, compute_cpu, max_value) = if refresh_kind.cpu() {
+            if self.need_processors_update {
+                self.refresh_processors(true);
+            }
 
-        let (total_time, compute_cpu, max_value) = if self.processors.is_empty() {
-            sysinfo_debug!("cannot compute processes CPU usage: no processor found...");
-            (0., false, 0.)
+            if self.processors.is_empty() {
+                sysinfo_debug!("cannot compute processes CPU usage: no processor found...");
+                (0., false, 0.)
+            } else {
+                let (new, old) = get_raw_times(&self.global_processor);
+                let total_time = if old > new { 1 } else { new - old };
+                (
+                    total_time as f32 / self.processors.len() as f32,
+                    true,
+                    self.get_max_process_cpu_usage(),
+                )
+            }
         } else {
-            let (new, old) = get_raw_times(&self.global_processor);
-            let total_time = if old > new { 1 } else { new - old };
-            (
-                total_time as f32 / self.processors.len() as f32,
-                true,
-                self.get_max_process_cpu_usage(),
-            )
+            (0., false, 0.)
         };
 
+        // FIXME: once `retain_mut` has been stabilized, remove `to_delete`.
         let mut to_delete = Vec::with_capacity(20);
 
         for (pid, proc_) in &mut self.process_list.tasks {
-            if !has_been_updated(proc_) {
+            if !proc_.updated {
                 to_delete.push(*pid);
             } else if compute_cpu {
                 compute_cpu_usage(proc_, total_time, max_value);
             }
+            proc_.updated = false;
         }
         for pid in to_delete {
             self.process_list.tasks.remove(&pid);
@@ -165,6 +174,8 @@ impl System {
 
     fn refresh_processors(&mut self, only_update_global_processor: bool) {
         if let Ok(f) = File::open("/proc/stat") {
+            self.need_processors_update = false;
+
             let buf = BufReader::new(f);
             let mut i: usize = 0;
             let first = self.processors.is_empty();
@@ -290,16 +301,15 @@ impl SystemExt for System {
                 String::new(),
             ),
             processors: Vec::with_capacity(4),
-            page_size_kb: unsafe { sysconf(_SC_PAGESIZE) as u64 / 1024 },
+            page_size_kb: unsafe { sysconf(_SC_PAGESIZE) / 1024 } as _,
             components: Vec::new(),
             disks: Vec::with_capacity(2),
             networks: Networks::new(),
             users: Vec::new(),
             boot_time: boot_time(),
+            need_processors_update: true,
+            clock_cycle: unsafe { sysconf(_SC_CLK_TCK) } as _,
         };
-        if !refreshes.cpu() {
-            s.refresh_processors(false); // We need the processors to be filled.
-        }
         s.refresh_specifics(refreshes);
         s
     }
@@ -337,7 +347,7 @@ impl SystemExt for System {
         self.refresh_processors(false);
     }
 
-    fn refresh_processes(&mut self) {
+    fn refresh_processes_specifics(&mut self, refresh_kind: ProcessRefreshKind) {
         let uptime = self.uptime();
         if refresh_procs(
             &mut self.process_list,
@@ -345,13 +355,15 @@ impl SystemExt for System {
             self.page_size_kb,
             0,
             uptime,
-            get_secs_since_epoch(),
+            self.clock_cycle,
+            refresh_kind,
         ) {
-            self.clear_procs();
+            self.clear_procs(refresh_kind);
         }
+        self.need_processors_update = true;
     }
 
-    fn refresh_process(&mut self, pid: Pid) -> bool {
+    fn refresh_process_specifics(&mut self, pid: Pid, refresh_kind: ProcessRefreshKind) -> bool {
         let uptime = self.uptime();
         let found = match _get_process_data(
             &Path::new("/proc/").join(pid.to_string()),
@@ -359,7 +371,8 @@ impl SystemExt for System {
             self.page_size_kb,
             0,
             uptime,
-            get_secs_since_epoch(),
+            self.clock_cycle,
+            refresh_kind,
         ) {
             Ok((Some(p), pid)) => {
                 self.process_list.tasks.insert(pid, p);
@@ -369,18 +382,22 @@ impl SystemExt for System {
             Err(_) => false,
         };
         if found {
-            self.refresh_processors(true);
+            if refresh_kind.cpu() {
+                self.refresh_processors(true);
 
-            if self.processors.is_empty() {
-                sysinfo_debug!("Cannot compute process CPU usage: no processors found...");
-                return found;
-            }
-            let (new, old) = get_raw_times(&self.global_processor);
-            let total_time = (if old >= new { 1 } else { new - old }) as f32;
+                if self.processors.is_empty() {
+                    sysinfo_debug!("Cannot compute process CPU usage: no processors found...");
+                    return found;
+                }
+                let (new, old) = get_raw_times(&self.global_processor);
+                let total_time = (if old >= new { 1 } else { new - old }) as f32;
 
-            let max_cpu_usage = self.get_max_process_cpu_usage();
-            if let Some(p) = self.process_list.tasks.get_mut(&pid) {
-                compute_cpu_usage(p, total_time / self.processors.len() as f32, max_cpu_usage);
+                let max_cpu_usage = self.get_max_process_cpu_usage();
+                if let Some(p) = self.process_list.tasks.get_mut(&pid) {
+                    compute_cpu_usage(p, total_time / self.processors.len() as f32, max_cpu_usage);
+                }
+            } else if let Some(p) = self.process_list.tasks.get_mut(&pid) {
+                p.updated = false;
             }
         }
         found
@@ -391,7 +408,7 @@ impl SystemExt for System {
     }
 
     fn refresh_users_list(&mut self) {
-        self.users = crate::linux::users::get_users_list();
+        self.users = crate::users::get_users_list();
     }
 
     // COMMON PART
@@ -679,13 +696,6 @@ fn get_system_info_android(info: InfoType) -> Option<String> {
     }
 }
 
-fn get_secs_since_epoch() -> u64 {
-    match SystemTime::now().duration_since(SystemTime::UNIX_EPOCH) {
-        Ok(n) => n.as_secs(),
-        _ => panic!("SystemTime before UNIX EPOCH!"),
-    }
-}
-
 #[cfg(test)]
 mod test {
     #[cfg(target_os = "android")]
