@@ -1,8 +1,4 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2015 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 // Once https://github.com/rust-lang/rfcs/blob/master/text/1422-pub-restricted.md
 // feature gets stabilized, we can move common parts in here.
@@ -28,7 +24,6 @@ mod tests {
     fn test_refresh_process() {
         let mut sys = System::new();
         assert!(sys.processes().is_empty(), "no process should be listed!");
-        sys.refresh_processes();
         // We don't want to test on unsupported systems.
 
         #[cfg(not(feature = "apple-sandbox"))]
