@@ -1,15 +1,12 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2017 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use crate::{
     common::{Gid, Uid},
     sys::{Component, Disk, Networks, Process, Processor},
 };
 use crate::{
-    DiskType, DiskUsage, LoadAvg, NetworksIter, Pid, ProcessStatus, RefreshKind, Signal, User,
+    DiskType, DiskUsage, LoadAvg, NetworksIter, Pid, ProcessRefreshKind, ProcessStatus,
+    RefreshKind, Signal, User,
 };
 
 use std::collections::HashMap;
@@ -127,23 +124,38 @@ pub trait DiskExt: Debug {
 
 /// Contains all the methods of the [`Process`][crate::Process] struct.
 pub trait ProcessExt: Debug {
-    /// Creates a new process only containing the given information.
+    /// Sends [`Signal::Kill`] to the process (which is the only signal supported on all platforms
+    /// by this crate).
     ///
-    /// On windows, the `start_time` argument is ignored.
-    #[doc(hidden)]
-    fn new(pid: Pid, parent: Option<Pid>, start_time: u64) -> Self;
+    /// If you want to send another signal, take a look at [`ProcessExt::kill_with`].
+    ///
+    /// ```no_run
+    /// use sysinfo::{ProcessExt, System, SystemExt};
+    ///
+    /// let s = System::new();
+    /// if let Some(process) = s.process(1337) {
+    ///     process.kill();
+    /// }
+    /// ```
+    fn kill(&self) -> bool;
 
-    /// Sends the given `signal` to the process.
+    /// Sends the given `signal` to the process. If the signal doesn't exist on this platform,
+    /// it'll do nothing and will return `None`. Otherwise it'll return if the signal was sent
+    /// successfully.
+    ///
+    /// If you just want to kill the process, use [`ProcessExt::kill`] directly.
     ///
     /// ```no_run
     /// use sysinfo::{ProcessExt, Signal, System, SystemExt};
     ///
     /// let s = System::new();
     /// if let Some(process) = s.process(1337) {
-    ///     process.kill(Signal::Kill);
+    ///     if process.kill_with(Signal::Kill).is_none() {
+    ///         eprintln!("This signal isn't supported on this platform");
+    ///     }
     /// }
     /// ```
-    fn kill(&self, signal: Signal) -> bool;
+    fn kill_with(&self, signal: Signal) -> Option<bool>;
 
     /// Returns the name of the process.
     ///
@@ -286,18 +298,30 @@ pub trait ProcessExt: Debug {
     /// ```
     fn status(&self) -> ProcessStatus;
 
-    /// Returns the time of process launch (in seconds).
+    /// Returns the time where the process was started (in seconds) from epoch.
     ///
     /// ```no_run
     /// use sysinfo::{ProcessExt, System, SystemExt};
     ///
     /// let s = System::new();
     /// if let Some(process) = s.process(1337) {
-    ///     println!("Running since {} seconds", process.start_time());
+    ///     println!("Started at {} seconds", process.start_time());
     /// }
     /// ```
     fn start_time(&self) -> u64;
 
+    /// Returns for how much time the process has been running (in seconds).
+    ///
+    /// ```no_run
+    /// use sysinfo::{ProcessExt, System, SystemExt};
+    ///
+    /// let s = System::new();
+    /// if let Some(process) = s.process(1337) {
+    ///     println!("Running since {} seconds", process.run_time());
+    /// }
+    /// ```
+    fn run_time(&self) -> u64;
+
     /// Returns the total CPU usage (in %). Notice that it might be bigger than 100 if run on a
     /// multicore machine.
     ///
@@ -319,7 +343,7 @@ pub trait ProcessExt: Debug {
 
     /// Returns number of bytes read and written to disk.
     ///
-    /// /!\\ On Windows, this method actually returns **ALL** I/O read and written bytes.
+    /// ⚠️ On Windows and FreeBSD, this method actually returns **ALL** I/O read and written bytes.
     ///
     /// ```no_run
     /// use sysinfo::{ProcessExt, System, SystemExt};
@@ -407,7 +431,7 @@ pub trait ProcessorExt: Debug {
 }
 
 /// Contains all the methods of the [`System`][crate::System] type.
-pub trait SystemExt: Sized + Debug + Default {
+pub trait SystemExt: Sized + Debug + Default + Send + Sync {
     /// Returns `true` if this OS is supported. Please refer to the
     /// [crate-level documentation](index.html) to get the list of supported OSes.
     ///
@@ -483,12 +507,14 @@ pub trait SystemExt: Sized + Debug + Default {
     /// "refresh_" methods.
     ///
     /// ```
-    /// use sysinfo::{RefreshKind, System, SystemExt};
+    /// use sysinfo::{ProcessRefreshKind, RefreshKind, System, SystemExt};
     ///
     /// let mut s = System::new_all();
     ///
     /// // Let's just update networks and processes:
-    /// s.refresh_specifics(RefreshKind::new().with_networks().with_processes());
+    /// s.refresh_specifics(
+    ///     RefreshKind::new().with_networks().with_processes(ProcessRefreshKind::everything()),
+    /// );
     /// ```
     fn refresh_specifics(&mut self, refreshes: RefreshKind) {
         if refreshes.memory() {
@@ -507,8 +533,8 @@ pub trait SystemExt: Sized + Debug + Default {
         } else if refreshes.networks() {
             self.refresh_networks();
         }
-        if refreshes.processes() {
-            self.refresh_processes();
+        if let Some(kind) = refreshes.processes() {
+            self.refresh_processes_specifics(kind);
         }
         if refreshes.disks_list() {
             self.refresh_disks_list();
@@ -587,24 +613,54 @@ pub trait SystemExt: Sized + Debug + Default {
 
     /// Gets all processes and updates their information.
     ///
+    /// It does the same as `system.refresh_processes_specifics(ProcessRefreshKind::everything())`.
+    ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let mut s = System::new_all();
     /// s.refresh_processes();
     /// ```
-    fn refresh_processes(&mut self);
+    fn refresh_processes(&mut self) {
+        self.refresh_processes_specifics(ProcessRefreshKind::everything());
+    }
+
+    /// Gets all processes and updates the specified information.
+    ///
+    /// ```no_run
+    /// use sysinfo::{ProcessRefreshKind, System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    /// s.refresh_processes_specifics(ProcessRefreshKind::new());
+    /// ```
+    fn refresh_processes_specifics(&mut self, refresh_kind: ProcessRefreshKind);
 
     /// Refreshes *only* the process corresponding to `pid`. Returns `false` if the process doesn't
     /// exist. If it isn't listed yet, it'll be added.
     ///
+    /// It is the same as calling
+    /// `sys.refresh_process_specifics(pid, ProcessRefreshKind::everything())`.
+    ///
     /// ```no_run
     /// use sysinfo::{System, SystemExt};
     ///
     /// let mut s = System::new_all();
     /// s.refresh_process(1337);
     /// ```
-    fn refresh_process(&mut self, pid: Pid) -> bool;
+    fn refresh_process(&mut self, pid: Pid) -> bool {
+        self.refresh_process_specifics(pid, ProcessRefreshKind::everything())
+    }
+
+    /// Refreshes *only* the process corresponding to `pid`. Returns `false` if the process doesn't
+    /// exist. If it isn't listed yet, it'll be added.
+    ///
+    /// ```no_run
+    /// use sysinfo::{ProcessRefreshKind, System, SystemExt};
+    ///
+    /// let mut s = System::new_all();
+    /// s.refresh_process_specifics(1337, ProcessRefreshKind::new());
+    /// ```
+    fn refresh_process_specifics(&mut self, pid: Pid, refresh_kind: ProcessRefreshKind) -> bool;
 
     /// Refreshes the listed disks' information.
     ///
@@ -749,20 +805,26 @@ pub trait SystemExt: Sized + Debug + Default {
 
     /// Returns "global" processors information (aka the addition of all the processors).
     ///
-    /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// To have up-to-date information, you need to call [`SystemExt::refresh_cpu`] or
+    /// [`SystemExt::refresh_specifics`] with `cpu` enabled.
     ///
-    /// let s = System::new();
+    /// ```no_run
+    /// use sysinfo::{ProcessorExt, RefreshKind, System, SystemExt};
+    ///
+    /// let s = System::new_with_specifics(RefreshKind::new().with_cpu());
     /// println!("{}%", s.global_processor_info().cpu_usage());
     /// ```
     fn global_processor_info(&self) -> &Processor;
 
     /// Returns the list of the processors.
     ///
-    /// ```no_run
-    /// use sysinfo::{ProcessorExt, System, SystemExt};
+    /// By default, the list of processors is empty until you call [`SystemExt::refresh_cpu`] or
+    /// [`SystemExt::refresh_specifics`] with `cpu` enabled.
     ///
-    /// let s = System::new();
+    /// ```no_run
+    /// use sysinfo::{ProcessorExt, RefreshKind, System, SystemExt};
+    ///
+    /// let s = System::new_with_specifics(RefreshKind::new().with_cpu());
     /// for processor in s.processors() {
     ///     println!("{}%", processor.cpu_usage());
     /// }
