@@ -1,8 +1,4 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2015 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use crate::{DiskUsage, Pid, ProcessExt, ProcessStatus, Signal};
 
@@ -23,12 +19,12 @@ pub struct Process {
 }
 
 impl ProcessExt for Process {
-    fn new(pid: Pid, parent: Option<Pid>, _start_time: u64) -> Process {
-        Process { pid, parent }
+    fn kill(&self) -> bool {
+        false
     }
 
-    fn kill(&self, _signal: Signal) -> bool {
-        false
+    fn kill_with(&self, _signal: Signal) -> Option<bool> {
+        None
     }
 
     fn name(&self) -> &str {
@@ -79,6 +75,10 @@ impl ProcessExt for Process {
         0
     }
 
+    fn run_time(&self) -> u64 {
+        0
+    }
+
     fn cpu_usage(&self) -> f32 {
         0.0
     }
